@@ -29,19 +29,9 @@ USER_AGENT = "asusrouter--DUTUtil-"
 # Library defaults
 DEFAULT_CACHE_TIME = 5.0
 DEFAULT_SLEEP_TIME = 0.1
-DEFAULT_TIMEOUT = 10.0
+DEFAULT_TIMEOUT = 15
 DEFAULT_TIMEOUT_CONNECTION = 180.0
 
-DEFAULT_TIMEOUTS = [
-    5.0,
-    10.0,
-    20.0,
-    30.0,
-    60.0,
-    120.0,
-    180.0,
-]
-
 DEFAULT_RESULT_SUCCESS = {"statusCode": "200"}
 
 UNKNOWN_MEMBER = -999
