@@ -8,16 +8,19 @@ from datetime import datetime, timedelta
 from typing import Any, Optional, Tuple
 
 import xmltodict
-from dateutil.parser import parse as dtparse
 
 from asusrouter.modules.data import AsusData, AsusDataState
 from asusrouter.modules.endpoint import data_get
 from asusrouter.modules.openvpn import AsusOVPNClient, AsusOVPNServer
 from asusrouter.tools.cleaners import clean_dict, clean_dict_key_prefix
-from asusrouter.tools.converters import safe_int
+from asusrouter.tools.converters import safe_datetime, safe_int
 from asusrouter.tools.readers import merge_dicts
 
-from .devicemap_const import DEVICEMAP_BY_INDEX, DEVICEMAP_BY_KEY, DEVICEMAP_CLEAR
+from .devicemap_const import (
+    DEVICEMAP_BY_INDEX,
+    DEVICEMAP_BY_KEY,
+    DEVICEMAP_CLEAR,
+)
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -32,7 +35,9 @@ def read(content: str) -> dict[str, Any]:
 
     # Parse the XML data
     try:
-        xml_content: dict[str, Any] = xmltodict.parse(content).get("devicemap", {})
+        xml_content: dict[str, Any] = xmltodict.parse(content).get(
+            "devicemap", {}
+        )
         if not xml_content:
             _LOGGER.debug("Received empty devicemap XML")
             return devicemap
@@ -56,16 +61,18 @@ def read(content: str) -> dict[str, Any]:
             # If the key is not in the devicemap, continue
             if key not in devicemap[output_group]:
                 continue
-            devicemap[output_group][key] = devicemap[output_group][key].replace(
-                clear_value, ""
-            )
+            devicemap[output_group][key] = devicemap[output_group][
+                key
+            ].replace(clear_value, "")
 
     # Clean the devicemap values
     devicemap = clean_dict(devicemap)
 
     # Clean the devicemap values_dict from output_group prefix
     for output_group, values_dict in devicemap.items():
-        devicemap[output_group] = clean_dict_key_prefix(values_dict, output_group)
+        devicemap[output_group] = clean_dict_key_prefix(
+            values_dict, output_group
+        )
 
     # Return the devicemap
     return devicemap
@@ -156,28 +163,30 @@ def read_special(xml_content: dict[str, Any]) -> dict[str, Any]:
     return {}
 
 
-def read_uptime_string(content: str) -> datetime | None:
+def read_uptime_string(
+    content: str,
+) -> Tuple[Optional[datetime], Optional[int]]:
     """Read uptime string and return proper datetime object."""
 
     # Split the content into the date/time part and the seconds part
     uptime_parts = content.split("(")
     if len(uptime_parts) < 2:
-        return None
+        return (None, None)
 
     # Extract the number of seconds from the seconds part
     seconds_match = re.search("([0-9]+)", uptime_parts[1])
     if not seconds_match:
-        return None
+        return (None, None)
+    seconds = safe_int(seconds_match.group())
 
-    try:
-        seconds = int(seconds_match.group())
-        when = dtparse(uptime_parts[0])
-    except ValueError:
-        return None
+    when = safe_datetime(uptime_parts[0])
+    if when is None:
+        return (None, seconds)
 
+    # This part will always work, since seconds are always an integer
+    # unless `safe_int` got broken
     uptime = when - timedelta(seconds=seconds)
-
-    return uptime
+    return (uptime, seconds)
 
 
 def process(data: dict[str, Any]) -> dict[AsusData, Any]:
@@ -190,7 +199,9 @@ def process(data: dict[str, Any]) -> dict[AsusData, Any]:
     devicemap = data
 
     # Boot time
-    prev_boottime_object: Optional[AsusDataState] = history.get(AsusData.BOOTTIME)
+    prev_boottime_object: Optional[AsusDataState] = history.get(
+        AsusData.BOOTTIME
+    )
     prev_boottime = prev_boottime_object.data if prev_boottime_object else None
     boottime, reboot = process_boottime(devicemap, prev_boottime)
 
@@ -219,19 +230,19 @@ def process_boottime(
     # Reboot flag
     reboot = False
 
-    boottime = {}
+    boottime: dict[str, Any] = {}
 
-    # Since precision is 1 second, could be that old and new are 1 sec different.
-    # In this case, we should not change the boot time,
-    # but keep the previous value to avoid regular changes
+    # Since precision is 1 second, could be that old and new
+    # are 1 sec different. In this case, we should not change
+    # the boot time, but keep the previous value to avoid regular changes
     sys = devicemap.get("sys")
     if sys:
         uptime_str = sys.get("uptimeStr")
+        _LOGGER.debug("Uptime string: %s", uptime_str)
         if uptime_str:
-            time = read_uptime_string(uptime_str)
+            time, seconds = read_uptime_string(uptime_str)
             if time:
                 boottime["datetime"] = time
-
                 if prev_boottime and "datetime" in prev_boottime:
                     delta = time - prev_boottime["datetime"]
 
@@ -241,6 +252,8 @@ def process_boottime(
                     else:
                         boottime = prev_boottime
 
+                boottime["uptime"] = seconds
+
     return boottime, reboot
 
 
