@@ -81,6 +81,7 @@ class AsusDevice:  # pylint: disable=too-many-instance-attributes
     # Flags for device features
     aura: bool = False
     aura_zone: int = 0
+    dsl: bool = False
     led: bool = False
     ookla: bool = False
     vpn_status: bool = False
@@ -121,12 +122,19 @@ async def collect_identity(
     endpoints, onboarding = await _check_endpoints(api_query)
     identity["endpoints"] = endpoints
     _LOGGER.debug("Endpoints checked")
+    # Manually assign Aura endpoint value
+    identity["endpoints"][EndpointTools.AURA] = identity.get("aura", False)
 
     # Check onboarding to get nice model name
     this_device = onboarding.get(identity["mac"])
     if isinstance(this_device, AiMeshDevice):
         identity["model"] = this_device.model
 
+    # Check if DSL
+    if endpoints[Endpoint.DSL] is True:
+        identity["dsl"] = True
+        _LOGGER.debug("DSL-compatible device detected")
+
     # Check if Merlin
     if endpoints[Endpoint.SYSINFO] is True:
         identity["merlin"] = True
